#!/usr/bin/env python3
# --------------------( LICENSE                            )--------------------
# Copyright 2017-2018 by Alexis Pietak & Cecil Curry.
# See "LICENSE" for further details.

'''
:mod:`PySide2`-based file functionality.
'''

# ....................{ IMPORTS                            }....................
from PySide2.QtCore import QCoreApplication
from PySide2.QtWidgets import QFileDialog
# from betse.util.io.log import logs
from betse.lib.pil import pils
from betse.lib.yaml import yamls
from betse.util.type.types import type_check, StrOrNoneTypes

# ....................{ GLOBALS                            }....................
_YAML_LABEL_TO_FILETYPES = {'YAML files': yamls.FILETYPES,}
'''
Dictionary mapping from a human-readable label to be displayed for each
iterable of YAML-specific filetypes accepted by this dialog to that iterable.
'''

# ....................{ SELECTORS ~ read                   }....................
@type_check
def select_file_read(*args, **kwargs) -> StrOrNoneTypes:
    '''
    Display a dialog requesting the user to select an existing file to be
    subsequently opened for reading (rather than overwriting), returning the
    absolute or relative filename of this file if this dialog was not cancelled
    *or* ``None`` otherwise (i.e., if this dialog was cancelled).

    Parameters
    ----------
    All paremeters are passed as is to the :func:`guipath.select_path`
    function. Note that:

    * The ``dialog_title`` parameter *must* be passed by the caller.
    * The ``dialog_callable`` parameter must *not* be passed by the caller.

    Returns
    ----------
    StrOrNoneTypes
        Either:
        * If this dialog was confirmed, the absolute filename of this file.
        * If this dialog was cancelled, ``None``.
    '''

    # Avoid circular import dependencies.
    from betsee.util.path import guipath

    # Return the user-based result of displaying this path dialog.
    return guipath.select_path(
        *args, dialog_callable=QFileDialog.getOpenFileName, **kwargs)


@type_check
def select_file_yaml_read(*args, **kwargs) -> StrOrNoneTypes:
    '''
    Display a dialog requesting the user to select an existing YAML file to be
    subsequently opened for reading (rather than overwriting), returning the
    absolute or relative filename of this file if this dialog was not cancelled
    *or* ``None`` otherwise (i.e., if this dialog was cancelled).

    See Also
    ----------
    :func:`select_file_read`
        Further details.
    '''

    return select_file_read(
        *args, label_to_filetypes=_YAML_LABEL_TO_FILETYPES, **kwargs)


#FIXME: Ideally, this dialog would also display a preview of the currently
#selected image. Unfortunately, as the "QFileDialog" class provides no default
#means of doing so, doing so is non-trivial. Indeed, while existing solutions do
#exist, they exist only for Qt < 5.9. Qt >= 5.9 internally changed the widget
#substructure of "QFileDialog" widgets from a grid to vertical box layout. It
#remains unclear just how both layouts could reasonably be supported.
#Nonetheless, for Qt < 5.9 code in PyQt5 and C++ implementing this preview, see:
#
#    http://www.qtcentre.org/threads/68998-Qt5-QFileDialog-with-preview-picture
#    http://www.qtcentre.org/threads/33593-How-can-i-have-a-QFileDialog-with-a-preview-of-the-picture?p=247022#post247022
@type_check
def select_image_read(*args, **kwargs) -> StrOrNoneTypes:
    '''
    Display a dialog requesting the user to select an existing image file to be
    subsequently opened for reading (rather than overwriting), returning the
    absolute or relative filename of this file if this dialog was not cancelled
    *or* ``None`` otherwise (i.e., if this dialog was cancelled).

    For generality, this dialog recognizes all image filetypes recognized by the
    third-party image processing framework leveraged by BETSE itself: Pillow.
    BETSE defers to this framework for most low-level image I/O operations.
    Deferring to the same framework guarantees parity with BETSE behaviour.

    See Also
    ----------
    :func:`select_file_read`
        Further details.
    '''

    # If no title was passed, default to a sensible title.
    if 'dialog_title' not in kwargs:
        kwargs['dialog_title'] = QCoreApplication.translate(
            'select_image_read', 'Select Image')

    # Select an image for reading and return the filename of this image.
    return select_file_read(
        *args,
        label_to_filetypes={'Image files': pils.get_filetypes(),},
        **kwargs)

# ....................{ SELECTORS ~ save                   }....................
@type_check
def select_file_save(*args, **kwargs) -> StrOrNoneTypes:
    '''
    Display a dialog requesting the user to select an arbitrary file (either
    existing or non-existing) to be subsequently opened for in-place saving and
    hence overwriting, returning the absolute filename of this file if this
    dialog was not cancelled *or* ``None`` otherwise (i.e., if this dialog was
    cancelled).

    If this file already exists, this dialog additionally requires the user to
    accept the subsequent overwriting of this file.

    See Also
    ----------
    :func:`select_file_read`
        Further details.
    '''

    # Avoid circular import dependencies.
    from betsee.util.path import guipath

    # Return the user-based result of displaying this path dialog.
    return guipath.select_path(
        *args, dialog_callable=QFileDialog.getSaveFileName, **kwargs)


@type_check
def select_file_yaml_save(*args, **kwargs) -> StrOrNoneTypes:
    '''
    Display a dialog requesting the user to select an existing YAML file to be
    subsequently opened for in-place saving and hence overwriting, returning the
    absolute filename of this file if this dialog was not cancelled *or* ``None``
    otherwise (i.e., if this dialog was cancelled).

    See Also
    ----------
    :func:`select_file_save`
        Further details.
    '''

    return select_file_save(
        *args, label_to_filetypes=_YAML_LABEL_TO_FILETYPES, **kwargs)



#!/usr/bin/env python3
# --------------------( LICENSE                            )--------------------
# Copyright 2017-2018 by Alexis Pietak & Cecil Curry.
# See "LICENSE" for further details.

'''
Submodule providing the startup splash progress visualizing the current progress
of this application loading.
'''

# ....................{ IMPORTS                            }....................
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# WARNING: To raise human-readable exceptions on application startup, the
# top-level of this module may import *ONLY* from submodules guaranteed to:
# * Exist, including standard Python, PySide2, and BETSE, and BETSEE modules.
#   This excludes all other third-party modules (mandatory or optional), whose
#   importability has yet to be validated.
# * Never raise exceptions on importation (e.g., due to module-level logic).
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

from PySide2.QtCore import Qt, QPixmap
from PySide2.QtWidgets import QSplashScreen
from betse.util.type.types import type_check
from betsee.util.app import guiapp

# ....................{ GLOBALS                            }....................
# This global is initialized by the make_splash() function called elsewhere.
GUI_APP_SPLASH = None
'''
:class:`QBetseeSplashScreen` singleton widget for this application.
'''

# ....................{ CLASSES                            }....................
class QBetseeSplashScreen(QSplashScreen):
    '''
    :class:`QSplashScreen`-based widget displaying a non-modal splash screen,
    typically used to present a multi-threaded graphical progress bar during
    time-consuming application startup.
    '''

    # ..................{ INITIALIZERS                       }..................
    @type_check
    def __init__(self, image_uri: str) -> None:
        '''
        Initialize this splash screen with the passed properties.

        Parameters
        ----------
        image_uri : str
            Qt-specific Uniform Resource Identifier (URI) of the single image to
            be displayed by this splash screen (e.g., ``://image/splash.svg``).
        '''

        # Initialize our superclass.
        super().__init__()

        # Application singleton, localized to avoid retaining references.
        gui_app = guiapp.get_app()

        #FIXME: Does this *REALLY* have to be rasterized and hence reduced to a
        #bitmap or can an SVG URI be encapsulated properly... somehow? The
        #answer currently appears to be "Nope!"

        # Display an in-memory image loaded from this on-disk URI.
        self.setPixmap(QPixmap(image_uri))

        # Prevent a frame (i.e., border) from being displayed around this splash
        # screen, thus displaying this screen as a "frameless window."
        self.setWindowFlags(self.windowFlags() | Qt.FramelessWindowHint)

        #FIXME: Is this actually required?
        # Prevent a background from being displayed under this splash screen,
        # preserving the alpha layer (i.e., transparency) in this image if any.
        self.setAutoFillBackground(False)

        # Display this splash screen *AFTER* setting all settings above.
        self.show()

        # Manually handle all outstanding GUI events. Since the main event loop
        # has yet to be started (i.e., by calling GUI_APP._exec()), event
        # handling *MUST* be performed manually.
        gui_app.processEvents()

    # ..................{ SETTERS                            }..................
    @type_check
    def set_info(self, info: str) -> None:
        '''
        Display the passed human-readable single-line string as the current
        message for this splash screen, replacing the prior such string if any.
        '''

        # Application singleton, localized to avoid retaining references.
        gui_app = guiapp.get_app()

        # Display this message.
        self.showMessage(info)

        # Manually handle all outstanding GUI events. See the __init__() method.
        gui_app.processEvents()



#!/usr/bin/env python3
# --------------------( LICENSE                            )--------------------
# Copyright 2017-2018 by Alexis Pietak & Cecil Curry.
# See "LICENSE" for further details.

'''
General-purpose :mod:`QTextEdit` widget subclasses.
'''

# ....................{ IMPORTS                            }....................
from PySide2.QtCore import Signal, Slot
from PySide2.QtWidgets import QPlainTextEdit

# ....................{ SUBCLASSES                         }....................
class QBetseePlainTextEdit(QPlainTextEdit):
    '''
    :mod:`QPlainTextEdit`-based widget optimized for intelligent display of
    plaintext (rather than rich text or hypertext).

    This application-specific widget augments the stock :class:`QPlainTextEdit`
    widget with additional support for intelligent text appending, including:

    * **Auto-scrolling,** automatically scrolling this widget to the most
      recently appended text.
    * **Thread-safe appending,** permitting different threads other than the
      thread owning this widget to safely append text to this widget via a
      preconfigured text appending signal and slot.
    '''

    # ..................{ INITIALIZERS                       }..................
    def __init__(self, *args, **kwargs) -> None:

        # Initialize our superclass with all passed parameters.
        super().__init__(*args, **kwargs)

        # Connect the text appending signal to the corresponding slot.
        self.append_text_signal.connect(self.append_text)

    # ..................{ SIGNALS                            }..................
    append_text_signal = Signal(str)
    '''
    Signal accepting a single string, connected to a slot of the same widget at
    widget initialization time to intelligently append this string to this
    widget.

    This signal is connected to the :meth:`append_text` slot at widget
    initialization time, enabling callers in different threads to thread-safely
    append text to this widget.
    '''

    # ..................{ SLOTS                              }..................
    @Slot(str)
    def append_text(self, text: str) -> None:
        '''
        Append the passed plain text to the text currently displayed by this
        widget and scroll this widget to display that text.

        This slot is connected to the :attr:`append_text_signal` signal at
        widget initialization time, permitting callers in different threads to
        thread-safely append text to this widget.

        Parameters
        ----------
        text : str
            Text to be appended.
        '''

        # Append this text to this widget.
        self.appendPlainText(text)

        # Scroll this widget to display this text.
        self.ensureCursorVisible()
